//-----------------------------------------------------------------------------------------
/// \file	RenderRoutines.h
/// \author	Rob Bateman
///	\date	23-10-04
/// \brief	These are simple openGL drawing routines for a few basic primitives that i seem
/// 		to use time and again over the course of wanging locators into Maya. Whilst
///			drawing geometry with glBegin / glEnd is relatively simple, it is also very
/// 		repetetive! To save a bit of time, i want the more common geometry primitives
///			to be wrapped up in slightly higher level functions.
///
//-----------------------------------------------------------------------------------------

#ifndef __RENDER_ROUTINES__H__
#define __RENDER_ROUTINES__H__

	#ifdef WIN32
		#pragma once
		// this is needed before any of the maya includes. Generally it is better to
		// set this within the preprocessor settings of visual C++.
		#define NT_PLUGIN
	#endif

	#include <maya/MPxLocatorNode.h>
	#include <maya/MString.h>
	#include <maya/MTypeId.h>
	#include <maya/MVector.h>
	#include <maya/MDataBlock.h>
	#include <maya/MDataHandle.h>
	#include <maya/M3dView.h>
	#include <maya/MDistance.h>
	#include <maya/MFnUnitAttribute.h>
	#include <maya/MFnNumericAttribute.h>
	#include <maya/MFnMessageAttribute.h>
	#include <maya/MFnTypedAttribute.h>
	#include <maya/MPxManipContainer.h>
	#include <maya/MPointArray.h>
	#include <math.h>

	class RenderRoutines
		: public MPxLocatorNode {
	public:

		RenderRoutines() : MPxLocatorNode() {};

		virtual ~RenderRoutines() {};

		/// \brief	this function can tell maya whether the locator node has a volume
		/// \return	true if bounded, false otherwise.
		///
		virtual bool isBounded() const;


		virtual void draw( M3dView& view,
								const MDagPath& DGpath,
								M3dView::DisplayStyle style,
								M3dView::DisplayStatus status );

		virtual void OnDraw( M3dView::DisplayStyle style,
							 M3dView::DisplayStatus status ) {
		}

	protected:

		/// \brief	Applies the user defined colour for the specified state
		void ApplyStateColour(const float intensity,M3dView::DisplayStatus status);

		//	These functions draw a circle on the specified plane
		//	\param	r		-	the radius of the circle
		//	\param	divs	-	the number of divisions around the circle
		//
		void DrawCircleXY(const float r,const int divs);
		void DrawCircleXZ(const float r,const int divs);
		void DrawCircleYZ(const float r,const int divs);

		void DrawFilledCircleXY(const float r,const int divs);
		void DrawFilledCircleXZ(const float r,const int divs);
		void DrawFilledCircleYZ(const float r,const int divs);


		//	These functions draw a quad in the specified plane.
		//	\brief	w	-	the width
		//	\brief	h	-	the height
		//
		void DrawQuadXY(const float w,const float h);
		void DrawQuadXZ(const float w,const float h);
		void DrawQuadYZ(const float w,const float h);

		void DrawFilledQuadXY(const float w,const float h);
		void DrawFilledQuadXZ(const float w,const float h);
		void DrawFilledQuadYZ(const float w,const float h);


		//	These functions draw a 2 crossed lines in the specified plane.
		//	\brief	w	-	the width
		//	\brief	h	-	the height
		//
		void DrawCrossXY(const float w,const float h);
		void DrawCrossXZ(const float w,const float h);
		void DrawCrossYZ(const float w,const float h);

		//	These functions draw an arc on the specified plane
		//	\param	r		-	the radius of the circle
		//	\param	sa		-	the start angle in degrees
		// 	\param	ea		-	the end angle in degrees
		//	\param	divs	-	the number of divisions around the circle
		//
		void DrawArcXY(const float r,const float sa,const float ea,const int divs);
		void DrawArcXZ(const float r,const float sa,const float ea,const int divs);
		void DrawArcYZ(const float r,const float sa,const float ea,const int divs);


		//	These functions draw an arc on the specified plane
		//	\param	sr		-	the start radius of the spiral
		//	\param	er		-	the end radius of the spiral
		//	\param	sa		-	the start angle in degrees
		// 	\param	ea		-	the end angle in degrees
		//	\param	h		-	the height of the spiral to draw
		//	\param	divs	-	the number of divisions around the circle
		//
		void DrawSpiralXY(const float sr,const float er,const float sa,const float ea,const float h,const int divs);
		void DrawSpiralXZ(const float sr,const float er,const float sa,const float ea,const float h,const int divs);
		void DrawSpiralYZ(const float sr,const float er,const float sa,const float ea,const float h,const int divs);


		void DrawSphere(const float r);
		void DrawCone(const float r,const float h);
		void DrawCube(const float w,const float h,const float d);
		void DrawFilledCube(const float w,const float h,const float d);

		//	These functions draw a 2 crossed lines in the specified plane.
		//	\brief	r	-	the radius
		//	\brief	h	-	the height
		//
		void DrawCylinderXY(const float r,const float h);
		void DrawCylinderXZ(const float r,const float h);
		void DrawCylinderYZ(const float r,const float h);


	};


#endif

