
#include "SquareTarget.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId SquareTarget::typeId( 0x7001B );

/// the typename of our node
const MString SquareTarget::typeName( "squareTarget" );


/// the locator dimension attributes
MObject SquareTarget::m_Size;


//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void SquareTarget::OnDraw(M3dView::DisplayStyle style,
						  M3dView::DisplayStatus status )
{
	// read attributes from this node
	MFnDagNode fn( thisMObject()  );

	float size;
	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );


	MColor outlineCol = colorRGB(status);

	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kLead )
		glColor3f(0.0f,1.0f,0.0f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

	DrawQuadXZ(size,size);


	if(status == M3dView::kTemplate || status == M3dView::kActiveTemplate )
		glColor4f(outlineCol.r,outlineCol.g,outlineCol.b,0.3f);
	else
		glColor4f(0.7f,0.7f,0.3f,0.3f);

	// disable depth testing and enable alpha blending.
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

		// prevent the quad from writing to the z buffer.
		glDepthMask(0);

			DrawFilledQuadXZ(size,size);

			glColor4f(0.9f,0.3f,0.0f,0.3f);
			DrawFilledQuadXZ(0.7f*size,0.7f*size);

			glColor4f(0.0f,0.0f,0.0f,0.3f);
			DrawFilledQuadXZ(0.3f*size,0.3f*size);

		// re-enable depth writes
		glDepthMask(1);

	glDisable(GL_BLEND);
}


//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox SquareTarget::boundingBox() const
{
	float size;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -size/2, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  size/2, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -size/2 ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  size/2 ) );

	return bbox;
}

//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* SquareTarget::creator()
{
	return new SquareTarget();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus SquareTarget::initialize()
{
	MFnUnitAttribute fn;

	MStatus stat;

	m_Size = fn.create( "size", "sz", MFnUnitAttribute::kDistance );

	// set a default value of 1 for the size
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_Size );

	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach size attribute to the squareTarget node");
		return stat;
	}


	return MS::kSuccess;
}
