
#include "VolumeBox.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId VolumeBox::typeId( 0x7001C );

/// the typename of our node
const MString VolumeBox::typeName( "volumeBox" );


/// the locator dimension attributes
MObject VolumeBox::m_xSize;
MObject VolumeBox::m_ySize;
MObject VolumeBox::m_zSize;

//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void VolumeBox::OnDraw( M3dView::DisplayStyle style,
						M3dView::DisplayStatus status )
{
	float xSize,ySize,zSize;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug xSizePlug  = fn.findPlug( m_xSize );
	xSizePlug.getValue( xSize );

	MPlug ySizePlug  = fn.findPlug( m_ySize );
	ySizePlug.getValue( ySize );

	MPlug zSizePlug = fn.findPlug( m_zSize );
	zSizePlug.getValue( zSize );


	// the MPxLocator::colorRGB() function returns the user specified colour for
	// a given selection state. For example, by default a selected item is green;
	// the user may however change this within the user preferences and say change
	// this to pink. In order for our node to maintain continuity with the rest
	// of the users settings, we should use this colour value somewhere obvious
	// on our nodes.
	MColor outlineCol = colorRGB(status);

	if(status == M3dView::kLead )
		glColor3f(0.0f,1.0f,0.0f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

	DrawCube(xSize,ySize,zSize);
}

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox VolumeBox::boundingBox() const {

	float xSize,ySize,zSize;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );

	MPlug xSizePlug  = fn.findPlug( m_xSize );
	xSizePlug.getValue( xSize );

	MPlug ySizePlug  = fn.findPlug( m_ySize );
	ySizePlug.getValue( ySize );

	MPlug zSizePlug = fn.findPlug( m_zSize );
	zSizePlug.getValue( zSize );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -0.5f*xSize, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  0.5f*xSize, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,-0.5f*ySize,  0.0f ) );
	bbox.expand( MPoint(  0.0f, 0.5f*ySize,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -0.5f*zSize ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  0.5f*zSize ) );

	return bbox;
}


//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* VolumeBox::creator()
{
	return new VolumeBox();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus VolumeBox::initialize()
{
	// in maya, all attributes are individual objects. The nodes within maya
	// maintain their own internal list of attribute objects. Thankfully
	// we don't have to manipulate this list of attributes ourselves. We simply
	// have to create all of the attributes and then add them to the node.
	//
	// A number of function sets allow you to create attributes of various
	// types, You may want attributes of various types, ie time, colour,
	// integer, float, string, bool, array etc, etc...
	//
	// The most common function set for creating an attribute is
	// MFnNumericAttribute since it handles floats, ints etc. In this case
	// however we want to add attributes that represents the nodes size.
	// This should be done using the MFnUnitAttribute function set. The
	// reason for this is that sizes may change since maya's units can
	// be changed to inches, meters, feet etc.
 	//
	MFnUnitAttribute fn;

	MStatus stat;

	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_xSize = fn.create( "xSize", "xs", MFnUnitAttribute::kDistance );

	// set a default value of 1 for the size
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );

	// An attribute will only appear in the channel box if it is keyable.
	// this means we usually make most attributes keyable, even if we
	// never want to animate them. (since the channel box is handy for
	// setting values).
	//
	// If we have an attribute that we want in the channel box, but we do
	// not want it to be keyed; we can overide the MPxNode::legalConnection()
	// function to say that anim curves should not be attached to the
	// non keyable attribute.
	//
	fn.setKeyable( true );

	// set a minim value for the attribute
	fn.setMin( 0.1f );

	// finally add the attribute to this node.
	stat = addAttribute( m_xSize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach xSize attribute to the volumeBox node");
		return stat;
	}



	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_ySize = fn.create( "ySize", "ys", MFnUnitAttribute::kDistance );
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_ySize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach ySize attribute to the volumeBox node");
		return stat;
	}


	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_zSize = fn.create( "zSize", "zs", MFnUnitAttribute::kDistance );
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_zSize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach zSize attribute to the volumeBox node");
		return stat;
	}


	return MS::kSuccess;
}
