
#include "VolumeCube.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId VolumeCube::typeId( 0x7001E );

/// the typename of our node
const MString VolumeCube::typeName( "volumeCube" );


/// the locator dimension attributes
MObject VolumeCube::m_Size;



void VolumeCube::OnDraw(  M3dView::DisplayStyle style,
						  M3dView::DisplayStatus status ) {

	float size;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );

	// the MPxLocator::colorRGB() function returns the user specified colour for
	// a given selection state. For example, by default a selected item is green;
	// the user may however change this within the user preferences and say change
	// this to pink. In order for our node to maintain continuity with the rest
	// of the users settings, we should use this colour value somewhere obvious
	// on our nodes.
	MColor outlineCol = colorRGB(status);

	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kLead )
		glColor3f(0.0f,1.0f,0.0f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

	DrawCube(size,size,size);

}

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox VolumeCube::boundingBox() const
{

	float size;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -size/2, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  size/2, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,size/2,  0.0f ) );
	bbox.expand( MPoint(  0.0f,-size/2,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -size/2 ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  size/2 ) );

	return bbox;
}

//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* VolumeCube::creator()
{
	return new VolumeCube();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus VolumeCube::initialize()
{
	// in maya, all attributes are individual objects. The nodes within maya
	// maintain their own internal list of attribute objects. Thankfully
	// we don't have to manipulate this list of attributes ourselves. We simply
	// have to create all of the attributes and then add them to the node.
	//
	// A number of function sets allow you to create attributes of various
	// types, You may want attributes of various types, ie time, colour,
	// integer, float, string, bool, array etc, etc...
	//
	// The most common function set for creating an attribute is
	// MFnNumericAttribute since it handles floats, ints etc. In this case
	// however we want to add attributes that represents the nodes size.
	// This should be done using the MFnUnitAttribute function set. The
	// reason for this is that sizes may change since maya's units can
	// be changed to inches, meters, feet etc.
 	//
	MFnUnitAttribute fn;

	MStatus stat;

	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_Size = fn.create( "size", "sz", MFnUnitAttribute::kDistance );

	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_Size );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach size attribute to the volumeCube node");
		return stat;
	}


	return MS::kSuccess;
}
