
#include "VolumeTriggerBox.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId VolumeTriggerBox::typeId( 0x70022 );

/// the typename of our node
const MString VolumeTriggerBox::typeName( "volumeTriggerBox" );

/// the locator dimension attributes
MObject VolumeTriggerBox::m_xSize;
MObject VolumeTriggerBox::m_ySize;
MObject VolumeTriggerBox::m_zSize;

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox VolumeTriggerBox::boundingBox() const {

	float xSize,ySize,zSize;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );

	MPlug xSizePlug  = fn.findPlug( m_xSize );
	xSizePlug.getValue( xSize );

	MPlug ySizePlug  = fn.findPlug( m_ySize );
	ySizePlug.getValue( ySize );

	MPlug zSizePlug = fn.findPlug( m_zSize );
	zSizePlug.getValue( zSize );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -0.5f*xSize, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  0.5f*xSize, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,ySize,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -0.5f*zSize ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  0.5f*zSize ) );

	return bbox;
}



//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void VolumeTriggerBox::OnDraw( M3dView::DisplayStyle style,
						  M3dView::DisplayStatus status )
{
	float xSize,ySize,zSize;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );

	MPlug xSizePlug  = fn.findPlug( m_xSize );
	xSizePlug.getValue( xSize );

	MPlug ySizePlug  = fn.findPlug( m_ySize );
	ySizePlug.getValue( ySize );

	MPlug zSizePlug = fn.findPlug( m_zSize );
	zSizePlug.getValue( zSize );


	glPushMatrix();
		// move this box up a bit
		glTranslatef(0,0.5f*ySize,0);

		MColor outlineCol = colorRGB(status);

		if(status == M3dView::kLead )
			glColor3f(0.0f,1.0f,0.0f);
		else
			// Otherwise i use a slightly varied intensity of the colour
			glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

		DrawCube(xSize,ySize,zSize);
	glPopMatrix();

	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kTemplate || status == M3dView::kActiveTemplate )
		glColor4f(outlineCol.r,outlineCol.g,outlineCol.b,0.3f);
	else
		glColor4f(0.7f,0.7f,0.3f,0.3f);

	// disable depth testing and enable alpha blending. This is
	//
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	// prevent the quad from writing to the z buffer.
	glDepthMask(0);

		DrawFilledQuadXZ(xSize,zSize);

		glTranslatef(0,ySize,0);
		DrawFilledQuadXZ(xSize,zSize);

	// re-enable depth writes
	glDepthMask(1);

	glDisable(GL_BLEND);
}


//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* VolumeTriggerBox::creator()
{
	return new VolumeTriggerBox();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus VolumeTriggerBox::initialize()
{
	MFnUnitAttribute fn;

	MStatus stat;

	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_xSize = fn.create( "xSize", "xs", MFnUnitAttribute::kDistance );

	// set a default value of 1 for the size
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );

	// An attribute will only appear in the channel box if it is keyable.
	// this means we usually make most attributes keyable, even if we
	// never want to animate them. (since the channel box is handy for
	// setting values).
	//
	// If we have an attribute that we want in the channel box, but we do
	// not want it to be keyed; we can overide the MPxNode::legalConnection()
	// function to say that anim curves should not be attached to the
	// non keyable attribute.
	//
	fn.setKeyable( true );

	// set a minim value for the attribute
	fn.setMin( 0.1f );

	// finally add the attribute to this node.
	stat = addAttribute( m_xSize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach xSize attribute to the volumeTriggerBox node");
		return stat;
	}



	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_ySize = fn.create( "ySize", "ys", MFnUnitAttribute::kDistance );
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_ySize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach ySize attribute to the volumeTriggerBox node");
		return stat;
	}


	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_zSize = fn.create( "zSize", "zs", MFnUnitAttribute::kDistance );
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_zSize );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach zSize attribute to the volumeTriggerBox node");
		return stat;
	}


	return MS::kSuccess;
}
