
#include "VolumeTriggerCone.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId VolumeTriggerCone::typeId( 0x70023 );

/// the typename of our node
const MString VolumeTriggerCone::typeName( "volumeTriggerCone" );

/// the locator dimension attributes
MObject VolumeTriggerCone::m_Radius;
MObject VolumeTriggerCone::m_Height;

//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void VolumeTriggerCone::OnDraw(M3dView::DisplayStyle style,
						  M3dView::DisplayStatus status )
{

	float radius,height;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug RadiusPlug = fn.findPlug( m_Radius );
	RadiusPlug.getValue( radius );

	MPlug HeightPlug  = fn.findPlug( m_Height );
	HeightPlug.getValue( height );

	glTranslatef(0,height/2,0);

	// the MPxLocator::colorRGB() function returns the user specified colour for
	// a given selection state. For example, by default a selected item is green;
	// the user may however change this within the user preferences and say change
	// this to pink. In order for our node to maintain continuity with the rest
	// of the users settings, we should use this colour value somewhere obvious
	// on our nodes.
	MColor outlineCol = colorRGB(status);


	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kLead )
		glColor3f(0.0f,1.0f,0.0f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

	DrawCone(radius,height);

	if(status == M3dView::kLead )
		glColor3f(0.2f,0.8f,0.2f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor3f(0.9f*outlineCol.r,0.9f*outlineCol.g,0.9f*outlineCol.b);


	glPushMatrix();
		glTranslatef(0,-0.5f*height,0);
		DrawCrossXZ(2*radius,2*radius);
	glPopMatrix();


	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kTemplate || status == M3dView::kActiveTemplate )
		glColor4f(outlineCol.r,outlineCol.g,outlineCol.b,0.3f);
	else
		glColor4f(0.7f,0.7f,0.3f,0.3f);

	// disable depth testing and enable alpha blending. This is
	//
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

		glDepthMask(0);

			glPushMatrix();
				glTranslatef( 0.0f,-0.5f*height, 0.0f );
				DrawFilledCircleXZ( radius, 64 );
			glPopMatrix();

		glDepthMask(1);
	glDisable(GL_BLEND);
}


//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox VolumeTriggerCone::boundingBox() const
{
	float radius,height;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );

	MPlug RadiusPlug = fn.findPlug( m_Radius );
	RadiusPlug.getValue( radius );

	MPlug HeightPlug = fn.findPlug( m_Height );
	HeightPlug.getValue( height );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -radius, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  radius, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,height,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -radius ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  radius ) );

	return bbox;
}


//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* VolumeTriggerCone::creator()
{
	return new VolumeTriggerCone();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus VolumeTriggerCone::initialize()
{
	// in maya, all attributes are individual objects. The nodes within maya
	// maintain their own internal list of attribute objects. Thankfully
	// we don't have to manipulate this list of attributes ourselves. We simply
	// have to create all of the attributes and then add them to the node.
	//
	// A number of function sets allow you to create attributes of various
	// types, You may want attributes of various types, ie time, colour,
	// integer, float, string, bool, array etc, etc...
	//
	// The most common function set for creating an attribute is
	// MFnNumericAttribute since it handles floats, ints etc. In this case
	// however we want to add attributes that represents the nodes size.
	// This should be done using the MFnUnitAttribute function set. The
	// reason for this is that sizes may change since maya's units can
	// be changed to inches, meters, feet etc.
 	//
	MFnUnitAttribute fn;

	MStatus stat;

	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_Radius = fn.create( "radius", "r", MFnUnitAttribute::kDistance );

	// set a default value of 1 for the size
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );

	// An attribute will only appear in the channel box if it is keyable.
	// this means we usually make most attributes keyable, even if we
	// never want to animate them. (since the channel box is handy for
	// setting values).
	//
	// If we have an attribute that we want in the channel box, but we do
	// not want it to be keyed; we can overide the MPxNode::legalConnection()
	// function to say that anim curves should not be attached to the
	// non keyable attribute.
	//
	fn.setKeyable( true );

	// set a minim value for the attribute
	fn.setMin( 0.1f );

	// finally add the attribute to this node.
	stat = addAttribute( m_Radius );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach radius attribute to the volumeTriggerCone node");
		return stat;
	}



	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_Height = fn.create( "height", "h", MFnUnitAttribute::kDistance );
	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_Height );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach height attribute to the volumeTriggerCone node");
		return stat;
	}



	return MS::kSuccess;
}
