
#include "VolumeTriggerCube.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId VolumeTriggerCube::typeId( 0x70024 );

/// the typename of our node
const MString VolumeTriggerCube::typeName( "volumeTriggerCube" );


/// the locator dimension attributes
MObject VolumeTriggerCube::m_Size;

//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void VolumeTriggerCube::OnDraw(M3dView::DisplayStyle style,
						  	   M3dView::DisplayStatus status )
{
	float size;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );


	// the MPxLocator::colorRGB() function returns the user specified colour for
	// a given selection state. For example, by default a selected item is green;
	// the user may however change this within the user preferences and say change
	// this to pink. In order for our node to maintain continuity with the rest
	// of the users settings, we should use this colour value somewhere obvious
	// on our nodes.
	MColor outlineCol = colorRGB(status);

	glPushMatrix();

		glTranslatef(0,size/2,0);

		// if the object is the last selected, there seems to be no colour for this.
		// therefore i check to see if the locator is the last selected item and then
		// just use green. I might just be doing something stupid though ;)
		//
		if(status == M3dView::kLead )
			glColor3f(0.0f,1.0f,0.0f);
		else
			// Otherwise i use a slightly varied intensity of the colour
			glColor3f(outlineCol.r,outlineCol.g,outlineCol.b);

		DrawCube(size,size,size);

	glPopMatrix();


	// if the object is the last selected, there seems to be no colour for this.
	// therefore i check to see if the locator is the last selected item and then
	// just use green. I might just be doing something stupid though ;)
	//
	if(status == M3dView::kTemplate || status == M3dView::kActiveTemplate )
		glColor4f(outlineCol.r,outlineCol.g,outlineCol.b,0.3f);
	else
		glColor4f(0.7f,0.7f,0.3f,0.3f);

	// disable depth testing and enable alpha blending. This is
	//
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

		// prevent the quad from writing to the z buffer.
		glDepthMask(0);

			glPushMatrix();
				glTranslatef(0,size,0);
				DrawFilledQuadXZ(size,size);
			glPopMatrix();

			DrawFilledQuadXZ(size,size);

		// re-enable depth writes
		glDepthMask(1);

	glDisable(GL_BLEND);
}


//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox VolumeTriggerCube::boundingBox() const
{
	float size;

	// we need to determine the size of the loocator node by reading back the
	// attributes contained on this node. To do this, we must attach a function
	// set to this node, and then read back the attributes.
	//
	MFnDagNode fn( thisMObject()  );


	MPlug SizePlug = fn.findPlug( m_Size );
	SizePlug.getValue( size );

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -size/2, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  size/2, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,size,  0.0f ) );
	bbox.expand( MPoint(  0.0f,0.0f,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -size/2 ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  size/2 ) );

	return bbox;
}

//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* VolumeTriggerCube::creator()
{
	return new VolumeTriggerCube();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus VolumeTriggerCube::initialize()
{
	MFnUnitAttribute fn;

	MStatus stat;

	// create a size attribute. We specify the long and short name
	// for the attribute, as well as the data type.
	m_Size = fn.create( "size", "sz", MFnUnitAttribute::kDistance );

	fn.setDefault( MDistance(1.0, MDistance::uiUnit()) );
	fn.setKeyable( true );
	fn.setMin( 0.1f );
	stat = addAttribute( m_Size );

	// check for error
	if( stat != MS::kSuccess ) {
		stat.perror("Unable to attach size attribute to the volumeTriggerCube node");
		return stat;
	}

	return MS::kSuccess;
}
